#include "Gamefield.h"
#include <SFML/Graphics.hpp>
#include <cmath>
#include <iostream>
#include <sstream>

bool g_youLostTheGame = false;
int g_score = 0;
float g_multiplier = 1.0f;

int main()
{
    static const float TICKTIME_START = 0.7f;
    static const int TICKTIME_DECREASE_START = 25;
    static const float TICKTIME_DECREASE_SCORE_MULTIPLIER = 2; //need to get n as much points as the last time to get next increase
    static const float TICKTIME_DECREASE_FACTOR = 0.75f;

    Gamefield g;
    if(!g.Init()) return 0;

    sf::Image roosterImg;
    if(!roosterImg.LoadFromFile("data/rooster.png"))
    {
        return 0;
    }

    //32 additional vertical pixels for score at bottom, 64 for rooster at the top
    sf::RenderWindow App(sf::VideoMode(Gamefield::WIDTH * Gamefield::CELLSIZE, Gamefield::HEIGHT * Gamefield::CELLSIZE + 96), "Year of the Rooster");

    //GregV and Joshyy reported an out-of-viewspace window. this should fix the problem.
    App.SetPosition(0, 0);

    float ticktime = TICKTIME_START;
    int nextTicktimeDecrease = TICKTIME_DECREASE_START;

    enum Gamestate
    {
        INTRO,
        GAME,
        SCORE
    };

    Gamestate curstate = INTRO;

    sf::String scorestring;
    scorestring.SetSize(16);
    scorestring.SetColor(sf::Color::Black);
    scorestring.SetPosition(8, App.GetHeight()-24);
    sf::String multiplierstring;
    multiplierstring.SetSize(16);
    multiplierstring.SetColor(sf::Color::Black);
    multiplierstring.SetPosition(App.GetWidth()/2, App.GetHeight()-24);

    sf::String string[5];
    string[0].SetStyle(sf::String::Bold);
    for(int i = 0; i < 5; ++i)
    {
        string[i].SetColor(sf::Color::Black);
        string[i].SetPosition(App.GetWidth()/2, App.GetHeight()/2 - 100 + 50*i);
    }

    string[0].SetText("Year of the Rooster");
    string[1].SetText("A Mini Ludum Dare 23 Game");
    string[2].SetText("by Mr. Wonko");
    string[4].SetText("Press enter to start");
    for(int i = 0; i < 5; ++i)
    {
        if(i == 3) continue;
        string[i].SetCenter(string[i].GetRect().GetWidth()/2, string[i].GetRect().GetHeight()/2);
    }

    sf::Shape roosterBG = sf::Shape::Rectangle(0, 0, App.GetWidth(), 64, sf::Color(85, 45, 20));

    sf::Sprite roosterSprite(roosterImg);
    roosterSprite.SetPosition(Gamefield::WIDTH/2*Gamefield::CELLSIZE, 0);

    sf::Clock c;

    while(App.IsOpened())
    {
        //event loop
        sf::Event e;
        while(App.GetEvent(e))
        {
            switch(e.Type)
            {
                case sf::Event::Closed:
                {
                    App.Close();
                    break;
                }
                case sf::Event::KeyPressed:
                {
                    switch(e.Key.Code)
                    {
                        case sf::Key::Escape:
                        {
                            App.Close();
                            break;
                        }
                        case sf::Key::Return:
                        {
                            if(curstate == INTRO)
                            {
                                curstate = GAME;
                                g.Start();
                            }
                            else if(curstate == SCORE)
                            {
                                //player wants another game - reset
                                g.Reset();
                                g_score = 0;
                                g_youLostTheGame = false;
                                ticktime = TICKTIME_START;
                                nextTicktimeDecrease = TICKTIME_DECREASE_START;
                                curstate = GAME;
                                g.Start();
                            }
                            break;
                        }
                        case sf::Key::Left:
                        {
                            if(curstate == GAME)
                            {
                                g.MoveLeft();
#ifdef _DEBUG
                                g.RenderToConsole();
#endif
                            }
                            break;
                        }
                        case sf::Key::Right:
                        {
                            if(curstate == GAME)
                            {
                                g.MoveRight();
#ifdef _DEBUG
                                g.RenderToConsole();
#endif
                            }
                            break;
                        }
                        default:
                        {
                            break;
                        }
                    }
                    break;
                }
                default:
                {
                    break;
                }
            }
        }

        switch(curstate)
        {

            case GAME:
            {
                while(g_score >= nextTicktimeDecrease)
                {
                    ticktime *= TICKTIME_DECREASE_FACTOR;
                    nextTicktimeDecrease *= TICKTIME_DECREASE_SCORE_MULTIPLIER;
                }

                //TODO: make the update interval smaller as the game goes on?
                if(c.GetElapsedTime() >= ticktime)
                {
                    g.Update();
                    c.Reset();
#ifdef _DEBUG
                    g.RenderToConsole();
#endif
                }

                {
                    std::stringstream ss;
                    ss<<g_score;
                    scorestring.SetText("Score: " + ss.str());
                }
                {
                    std::stringstream ss;
                    ss<<g_multiplier;
                    multiplierstring.SetText("Multiplier: " + ss.str());
                }

                App.Clear(sf::Color::White);
                g.RenderToTarget(App);
                App.Draw(roosterBG);
                App.Draw(roosterSprite);
                App.Draw(scorestring);
                App.Draw(multiplierstring);
                App.Display();

                if(g_youLostTheGame)
                {
                    curstate = SCORE;
                    std::stringstream ss;
                    ss<<g_score;
                    string[0].SetText("Game Over");
                    string[1].SetText("Your Score:");
                    string[2].SetText(ss.str());
                    string[3].SetText("Press enter to play again");
                    string[4].SetText("Press escape to Quit");
                    for(int i = 0; i < 5; ++i)
                    {
                        string[i].SetCenter(string[i].GetRect().GetWidth()/2, string[i].GetRect().GetHeight()/2);
                    }
                }
                break;
            }
            case INTRO:
            {
                App.Clear(sf::Color::White);
                for(int i = 0; i < 5; ++i)
                {
                    if(i == 3) continue;
                    App.Draw(string[i]);
                }
                App.Display();
                break;
            }
            case SCORE:
            {
                App.Clear(sf::Color::White);
                for(int i = 0; i < 5; ++i)
                {
                    App.Draw(string[i]);
                }
                App.Display();
                break;
            }
        }
    }
    return 0;
}
